<?php

namespace Oro\Bundle\EmailBundle\Tests\Unit\Form\Handler;

use Doctrine\ORM\EntityManager;
use Oro\Bundle\ConfigBundle\Config\ConfigChangeSet;
use Oro\Bundle\ConfigBundle\Config\ConfigManager;
use Oro\Bundle\EmailBundle\DependencyInjection\Configuration;
use Oro\Bundle\EmailBundle\Form\Handler\UserEmailConfigHandler;
use Oro\Bundle\UserBundle\Entity\User;
use Symfony\Component\Form\FormInterface;

class UserEmailConfigHandlerTest extends \PHPUnit_Framework_TestCase
{
    /** @var ConfigManager|\PHPUnit_Framework_MockObject_MockObject */
    private $configManager;

    /** @var ConfigChangeSet|\PHPUnit_Framework_MockObject_MockObject */
    private $changeSet;

    /** @var FormInterface|\PHPUnit_Framework_MockObject_MockObject */
    private $form;

    /** @var EntityManager|\PHPUnit_Framework_MockObject_MockObject */
    private $entityManager;

    /** @var UserEmailConfigHandler */
    private $handler;

    protected function setUp()
    {
        $this->entityManager = $this->createMock(EntityManager::class);
        $this->configManager = $this->createMock(ConfigManager::class);
        $this->changeSet = $this->createMock(ConfigChangeSet::class);
        $this->form = $this->createMock(FormInterface::class);

        $this->handler = new UserEmailConfigHandler($this->entityManager);
    }

    public function testHandleWithoutMailboxForm()
    {
        $this->form->expects($this->once())
            ->method('has')
            ->with($this->getConfigKey())
            ->willReturn(false);

        $this->assertNull($this->handler->handle($this->configManager, $this->changeSet, $this->form));
    }

    /**
     * @param mixed $data
     *
     * @dataProvider getHandleWithMailboxDataProvider
     */
    public function testHandleWithMailboxData($data)
    {
        $this->mockForm($data);

        $this->assertNull($this->handler->handle($this->configManager, $this->changeSet, $this->form));
    }

    /**
     * @return array
     */
    public function getHandleWithMailboxDataProvider()
    {
        return [
            'empty data' => [
                null
            ],
            'not array' => [
                new \stdClass()
            ],
            'empty array' => [
                []
            ],
            'without value key' => [
                ['key' => 'value']
            ],
            'without user instance' => [
                ['value' => new \stdClass()]
            ]
        ];
    }

    public function testHandle()
    {
        $user = new User();
        $data = [
            'value' => $user
        ];
        $this->mockForm($data);

        $this->entityManager->expects($this->once())
            ->method('persist')
            ->with($user);
        $this->entityManager->expects($this->once())
            ->method('flush');

        $this->handler->handle($this->configManager, $this->changeSet, $this->form);
    }

    /**
     * @return string
     */
    private function getConfigKey()
    {
        return Configuration::getConfigKeyByName('user_mailbox', ConfigManager::SECTION_VIEW_SEPARATOR);
    }

    /**
     * @param mixed $data
     */
    private function mockForm($data)
    {
        $mailboxForm = $this->createMock(FormInterface::class);
        $mailboxForm->expects($this->once())
            ->method('getData')
            ->willReturn($data);

        $this->form->expects($this->once())
            ->method('has')
            ->with($this->getConfigKey())
            ->willReturn(true);
        $this->form->expects($this->once())
            ->method('get')
            ->with($this->getConfigKey())
            ->willReturn($mailboxForm);
    }
}
